/* This file is part of mailfrom filter.
   Copyright (C) 2005, Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301 USA */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <mailutils/mailutils.h>
#include "mailfrom.h"

#ifdef USE_DBM

static char *dbname = DEFAULT_RATE_DATABASE;

struct rate_result {
	time_t timestamp;
	time_t interval;
	size_t count;
};

mf_status
get_rate(char *email, double *ret)
{
	mf_status rc;
	DBM_FILE db;
	DBM_DATUM key;
	DBM_DATUM contents;
	struct rate_result *rp, rate;
	double result;
	time_t t;
	
	debug1(50, "getting rate info for %s", email);
	if (mu_dbm_open(dbname, &db, MU_STREAM_RDWR, 0600)) {
		mu_error("mu_dbm_open(%s) failed: %s", dbname,
			 mu_strerror(errno));
		return mf_failure;
	}
	memset (&key, 0, sizeof key);
	memset (&contents, 0, sizeof contents);
	MU_DATUM_PTR (key) = email;
	MU_DATUM_SIZE (key) = strlen (email);
	t = time(NULL);
	if (mu_dbm_fetch(db, key, &contents) == 0) {
		rp = (struct rate_result *) MU_DATUM_PTR(contents);
		debug4(50,
		       "found time: %lu, interval: %lu, count: %lu, rate: %g",
		       rp->timestamp, rp->interval, rp->count,
		       rp->interval == 0 ? -1 :
		                      (double)rp->count/rp->interval);
	} else {
		/* Initialize the structure */
		rate.timestamp = t;
		rate.interval = 0;
		rate.count = 0;
		rp = &rate;
	}

	/* Update the structure */
	rp->interval = t - rp->timestamp + rp->interval;
	rp->timestamp = t;
	rp->count++;

	if (rp->interval == 0)
		result = 0;
	else
		result = (double) rp->count / rp->interval;
	debug2(50, "rate for %s is %g", email, result);

	/* Update the db */
	if (rates_expire_interval && rp->interval > rates_expire_interval) {
		debug1(50, "expiring %s rates", email);
		rp->interval = 0;
		rp->count = 1;
	} else 
		debug1(50, "updating %s rates", email);

	MU_DATUM_PTR(contents) = (void*)rp;
	MU_DATUM_SIZE(contents) = sizeof(*rp);
	if (mu_dbm_insert(db, key, contents, 1))
		mu_error ("Cannot insert datum");

	mu_dbm_datum_free(&contents);
	mu_dbm_close(db);
	*ret = result;
	return mf_success;
}

static void
print_rate_item(const char *email, size_t size, const void *content)
{
	const struct rate_result *res = content;
	double rate, exp_rate;
	char *str, *p;
	
	printf("%*.*s", size, size, email);
	if (size < 24) {
		size = 24 - size;
		printf("%*.*s", size, size, "");
	}
	if (res->interval) {
		rate = (double)res->count / res->interval;
		exp_rate = (double)(res->count + 1) /
			   (time(NULL) - res->timestamp + res->interval);
	} else 
		rate = exp_rate = -1;

	str = ctime(&res->timestamp);
	p = str + strlen (str) - 1;
	if (*p == '\n')
		*p = 0;
	
	printf(" %s %6lu %6lu %8.3g %8.3g",
	       str,
	       res->interval,
	       res->count,
	       rate,
	       exp_rate);
	
	if (predict_next_option) {
		time_t interval = (double)(res->count+1)/predict_rate - res->interval;
		time_t next_time = res->timestamp + interval;

		if (interval < 0)
			printf("; free to send\n");
		else
			printf("; in %lu sec. at %s",
			       interval, ctime(&next_time));
		
	} else
		printf("\n");
}

void
rate_list_item(char *email)
{
	db_list_item(dbname, email, print_rate_item);
}

void
rate_list_db()
{
	db_list(dbname, print_rate_item);
}

static int
rate_expire_item(const void *content)
{
	const struct rate_result *res = content;
	return rates_expire_interval && res->interval > rates_expire_interval;
}

void
rate_expire_db()
{
	db_expire(dbname, rate_expire_item);
}

void
rate_delete(char *email)
{
	debug1(50,"deleting rate info for %s", email);
	db_delete(dbname, email);
}

#endif
